<?php
class ama_unzip_Class {

    public function ama_unzip_file($file, $to) {

        @ini_set('memory_limit', '256M');

        $needed_dirs = array();
        $to = $this->ama_untrailingslashit($to);


        if (!is_dir($to)) { //Only do parents if no children exist
            $path = preg_split('![/\\\]!', $this->ama_untrailingslashit($to));
            for ($i = count($path); $i >= 0; $i--) {
                if (empty($path[$i]))
                    continue;

                $dir = implode('/', array_slice($path, 0, $i + 1));
                if (preg_match('!^[a-z]:$!i', $dir)) // Skip it if it looks like a Windows Drive letter.
                    continue;

                if (!is_dir($dir))
                    $needed_dirs[] = $dir;
                else
                    break; // A folder exists, therefor, we dont need the check the levels below this
            }
        }

        if (class_exists('ZipArchive') && apply_filters('unzip_file_use_ziparchive', true)) {
            $result = $this->ama_unzip_file_ziparchive($file, $to, $needed_dirs);
            if (true === $result) {
                return $result;
            } elseif (is_wp_error($result)) {
                if ('incompatible_archive' != $result->get_error_code())
                    return $result;
            }
        }
        // Fall through to PclZip if ZipArchive is not available, or encountered an error opening the file.
        return $this->ama_unzip_file_pclzip($file, $to, $needed_dirs);
    }

    public function ama_unzip_file_ziparchive($file, $to, $needed_dirs = array()) {

        $z = new ZipArchive();

        // PHP4-compat - php4 classes can't contain constants
        $zopen = $z->open($file, /* ZIPARCHIVE::CHECKCONS */ 4);
        if (true !== $zopen)
            return new WP_Error('incompatible_archive', __('Incompatible Archive.'));

        for ($i = 0; $i < $z->numFiles; $i++) {
            if (!$info = $z->statIndex($i))
                return new WP_Error('stat_failed', __('Could not retrieve file from archive.'));

            if ('__MACOSX/' === substr($info['name'], 0, 9)) // Skip the OS X-created __MACOSX directory
                continue;

            if ('/' == substr($info['name'], -1)) // directory
                $needed_dirs[] = $to . $this->ama_untrailingslashit($info['name']);
            else
                $needed_dirs[] = $to . $this->ama_untrailingslashit(dirname($info['name']));
        }

        $needed_dirs = array_unique($needed_dirs);
        foreach ($needed_dirs as $dir) {
            // Check the parent folders of the folders all exist within the creation array.
            if ($this->ama_untrailingslashit($to) == $dir) // Skip over the working directory, We know this exists (or will exist)
                continue;
            if (strpos($dir, $to) === false) // If the directory is not within the working directory, Skip it
                continue;

            $parent_folder = dirname($dir);
            while (!empty($parent_folder) && $this->ama_untrailingslashit($to) != $parent_folder && !in_array($parent_folder, $needed_dirs)) {
                $needed_dirs[] = $parent_folder;
                $parent_folder = dirname($parent_folder);
            }
        }
        asort($needed_dirs);

        // Create those directories if need be:
        foreach ($needed_dirs as $_dir) {
            echo '<br>' . $_dir . '<br>';
            if (!is_dir($_dir) && !mkdir($_dir, 0755)) // Only check to see if the Dir exists upon creation failure. Less I/O this way.
                return new WP_Error('mkdir_failed', __('Could not create directory.'), $_dir);
        }
        unset($needed_dirs);

        for ($i = 0; $i < $z->numFiles; $i++) {
            if (!$info = $z->statIndex($i))
                return new WP_Error('stat_failed', __('Could not retrieve file from archive.'));

            if ('/' == substr($info['name'], -1)) // directory
                continue;

            if ('__MACOSX/' === substr($info['name'], 0, 9)) // Don't extract the OS X-created __MACOSX directory files
                continue;

            $contents = $z->getFromIndex($i);
            if (false === $contents)
                return new WP_Error('extract_failed', __('Could not extract file from archive.'), $info['name']);

            if (!$this->ama_put_contents($to . $info['name'], $contents, 0755))
                return new WP_Error('copy_failed', __('Could not copy file.'), $to . $info['filename']);
        }

        $z->close();

        return true;
    }

    /**
     * This function should not be called directly, use unzip_file instead. Attempts to unzip an archive using the PclZip library.
     * Assumes that WP_Filesystem() has already been called and set up.
     *
     * @since 3.0.0
     * @see unzip_file
     * @access private
     *
     * @param string $file Full path and filename of zip archive
     * @param string $to Full path on the filesystem to extract archive to
     * @param array $needed_dirs A partial list of required folders needed to be created.
     * @return mixed WP_Error on failure, True on success
     */
    public function ama_unzip_file_pclzip($file, $to, $needed_dirs = array()) {

        require_once(ABSPATH . 'wp-admin/includes/class-pclzip.php');

        $archive = new PclZip($file);

        // Is the archive valid?
        if (false == ($archive_files = $archive->extract(PCLZIP_OPT_EXTRACT_AS_STRING)))
            return new WP_Error('incompatible_archive', __('Incompatible Archive.'), $archive->errorInfo(true));

        if (0 == count($archive_files))
            return new WP_Error('empty_archive', __('Empty archive.'));

        // Determine any children directories needed (From within the archive)
        foreach ($archive_files as $file) {
            if ('__MACOSX/' === substr($file['filename'], 0, 9)) // Skip the OS X-created __MACOSX directory
                continue;

            $needed_dirs[] = $to . $this->ama_untrailingslashit($file['folder'] ? $file['filename'] : dirname($file['filename']));
        }

        $needed_dirs = array_unique($needed_dirs);
        foreach ($needed_dirs as $dir) {
            // Check the parent folders of the folders all exist within the creation array.
            if ($this->ama_untrailingslashit($to) == $dir) // Skip over the working directory, We know this exists (or will exist)
                continue;
            if (strpos($dir, $to) === false) // If the directory is not within the working directory, Skip it
                continue;

            $parent_folder = dirname($dir);
            while (!empty($parent_folder) && $this->ama_untrailingslashit($to) != $parent_folder && !in_array($parent_folder, $needed_dirs)) {
                $needed_dirs[] = $parent_folder;
                $parent_folder = dirname($parent_folder);
            }
        }
        asort($needed_dirs);

        // Create those directories if need be:
        foreach ($needed_dirs as $_dir) {
            if (!is_dir($_dir) && !mkdir($_dir, 0755)) 
                return new WP_Error('mkdir_failed', __('Could not create directory.'), $_dir);
        }
        unset($needed_dirs);

        // Extract the files from the zip
        foreach ($archive_files as $file) {
            if ($file['folder'])
                continue;

            if ('__MACOSX/' === substr($file['filename'], 0, 9))
                continue;

            if (!$this->ama_put_contents($to . $file['filename'], $file['content'], 0755))
                return new WP_Error('copy_failed', __('Could not copy file.'), $to . $file['filename']);
        }
        return true;
    }

    public function ama_untrailingslashit($string) {
        return $string;
    }

    public function ama_put_contents($file, $contents, $mode = false) {
        if (!($fp = @fopen($file, 'w')))
            return false;
        @fwrite($fp, $contents);
        @fclose($fp);
        chmod($file, $mode);
        return true;
    }

}
function ama_upgrade_plugin_page() {
    ama_page_title();
    if (isset($_POST['upgrade_now'])) {
        ama_alter_table();
        if (strtolower($_FILES['upgradezip']['name']) == 'wp-amaz-one-plus.zip') {
            $Todir = ABSPATH . 'wp-content/plugins/';
            $upload_dir = ABSPATH . 'wp-content/plugins/' . plugin_basename(dirname(__FILE__)) . '/uploads/';
            if (!is_dir($upload_dir)) {
                @mkdir($upload_dir, 0777);
            }
            if (is_dir($upload_dir)) {
                @move_uploaded_file($_FILES['upgradezip']['tmp_name'], $upload_dir . $_FILES['upgradezip']['name']);
                $ama_unzip = new ama_unzip_Class();
                $result = $ama_unzip->ama_unzip_file($upload_dir . $_FILES['upgradezip']['name'], $Todir);
                @unlink($upload_dir . $_FILES['upgradezip']['name']);
                if ($result === true) {
                    ama_RedirectJs(admin_url('admin.php?page=ama_upgrade_success'));
                    exit;
                }
                if (is_wp_error($result)) {
                    echo '<div class="updated" id="message">';
                    foreach ($result->errors as $idx => $val) {
                        echo '<p>' . $val[0] . '</p>';
                    }
                    echo '</div>';
                }
            }
        } else if (empty($_FILES['upgradezip']['name'])) {
            echo '<div class="updated" id="message">';
            echo '<p>Please upload pulgin zip file.</p>';
            echo '</div>';
        } else {
            echo '<div class="updated" id="message">';
            echo '<p>Uploaded file is not valid plugin zip.</p>';
            echo '</div>';
        }
    }
    ?>

    <div class="wrap">
        <form action="admin.php?page=ama_upgrade_plugin" enctype="multipart/form-data" method="post">
            <h4>upload a plugin in .zip format</h4>
            <label for="pluginzip" class="screen-reader-text">Plugin zip file</label>
            <input type="file" name="upgradezip" id="upgradezip">
            <input type="submit" name="upgrade_now" value="Upgrade Now" class="button">
        </form>
    </div>
    <?php
}

function ama_upgrade_success_page() {
    ?>
    <div class="updated" id="message">
        <p><strong>WP-Amaz-On Plugin is updated successfully! </strong></p>
    </div>
    <?php
    ama_alter_table();
}

function ama_RedirectJs($url) {
    ?>
    <script type="text/javascript">
        window.location = '<?php echo $url ?>';
    </script>
    <?php
}
?>
